
"""
Front-end for generically connecting to databases that support the DB API.
"""

import urllib
import re

class ConnectionFactory:
    """
    Allows database connections to be created by an application without
    direct knowledge of the underlying database in use.
    """

    def parseURL(dburl):
        """
        Takes a JDBC-like URL that identifies a database connection. The
        identifier has the form:
          driver:hostname:port:username:password
        The fields in the identifier can be quoted with standard URL quoting
        to allow ':' characters etc. to be included in the fields.
        The return value is a tuple of the form:
          (driver, hostname, port, username, password)
        Note that the returned values are unquoted.
        """
        
        match = re.search('([^:]+?):([^:]+?):([^:]+?):([^:]+?):([^:]+):?(.*)$', dburl)

        driver = urllib.unquote( match.group(1) )
        hostname = urllib.unquote( match.group(2) )
        port = int(urllib.unquote( match.group(3) ))
        username = urllib.unquote( match.group(4) )
        password = urllib.unquote( match.group(5) )
        subname = urllib.unquote( match.group(6) )
        if match.lastgroup == 7:
            extra = urllib.unquote( match.group(7) )
        else:
            extra = None

        return (driver, hostname, port, username, password, subname, extra)
    
    parseURL = staticmethod(parseURL)

    def createConnection(self, dburl):
        """
        Create a database connection using the driver etc. specfied in the DB
        URL specified.
        """

        driver = dburl.split(':')[0]
        exec( 'from dbdrivers import %s' % driver )
        return eval( '%s.createConnection("%s")' % (driver, dburl) )
             

    def printURL(self, dburl):
        """
        Display the component parts of a DB URL.
        """
        (driver, hostname, port, username, password, subname, extra) = ConnectionFactory.parseURL(dburl)
        print 'Driver: ', driver
        print 'Host: ', hostname
        print 'Port: ', port
        print 'User: ', username
        print 'Password: ', password
        print 'Subname: ', subname
        print 'Extra: ', extra
        
if __name__ == '__main__':
    factory = ConnectionFactory()
    factory.printURL('oracle:localhost:1521:user:password:db')

    factory.createConnection('cx_oracle:localhost:1521:user:password:portal')
    factory.createConnection('sqlite:localhost:1521:user:password:%3amemory%3a')
